"use strict";
/**
 * @fileoverview Detects usages of legacy binding syntax
 * @author James Garbutt <htttps://github.com/43081j>
 */
//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------
const rule = {
    meta: {
        docs: {
            description: 'Detects usages of legacy binding syntax',
            category: 'Best Practices',
            url: 'https://github.com/43081j/eslint-plugin-lit/blob/master/docs/rules/template-legacy-binding-syntax.md'
        }
    },
    create(context) {
        // variables should be defined here
        const legacyPropertyPattern = /\b(on\-([\w\-]+)|([\w\-]+)([\$\?]))=$/;
        //----------------------------------------------------------------------
        // Helpers
        //----------------------------------------------------------------------
        //----------------------------------------------------------------------
        // Public
        //----------------------------------------------------------------------
        return {
            'TaggedTemplateExpression': (node) => {
                if (node.type === 'TaggedTemplateExpression' &&
                    node.tag.type === 'Identifier' &&
                    node.tag.name === 'html') {
                    for (const quasi of node.quasi.quasis) {
                        const val = quasi.value.raw;
                        const match = val.match(legacyPropertyPattern);
                        if (match) {
                            if (match[3]) {
                                let replacement = `${match[3]}=`;
                                if (match[4] === '?') {
                                    replacement = `?${match[3]}=`;
                                }
                                context.report({
                                    node: quasi,
                                    message: `Legacy lit-extended syntax is unsupported, did you mean to use "${replacement}"?`
                                });
                            }
                            else {
                                context.report({
                                    node: quasi,
                                    message: `Legacy lit-extended syntax is unsupported, did you mean to use "@${match[2]}="?`
                                });
                            }
                        }
                    }
                }
            }
        };
    }
};
module.exports = rule;
